# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SharedContent macro, based on the FeedReader macro

    @copyright: 2008, 2012, 2013, 2014 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.Page import Page
from MoinSupport import parseMacroArguments
from MoinShare import getUpdateSources, getUpdates, \
                      getUpdatesFromPage, getUpdatesFromStore, \
                      formatUpdate, Update, \
                      FeedMissingError, FeedContentTypeError

Dependencies = ["time"]

MAX_ENTRIES = 5

# The macro itself.

def execute(macro, args):
    request = macro.request
    fmt = macro.formatter
    _ = request.getText

    source_pages = []
    show_content = None
    max_entries = None

    for arg, value in parseMacroArguments(args):
        if arg == "sources":
            source_pages.append(value)
        elif arg == "show":
            show_content = value.lower()
        elif arg == "limit":
            try:
                max_entries = int(value)
            except ValueError:
                return fmt.text(_("SharedContent: limit must be set to the maximum number of entries to be shown"))

    if not source_pages:
        return fmt.text(_("SharedContent: at least one sources page must be specified"))

    sources = {}

    for source_page in source_pages:
        sources.update(getUpdateSources(source_page, request))

    if not sources:
        return fmt.text(_("SharedContent: at least one update source must be specified"))

    show_content = show_content or False
    max_entries = max_entries or MAX_ENTRIES

    # Retrieve updates, classifying them as missing or bad and excluding them if
    # appropriate.

    updates = []
    feeds = []
    unspecified = []
    missing = []
    bad_content = []

    for source_name, source_parameters in sources.items():
        location = source_parameters.get("location")
        if not location:
            unspecified.append(source_name)
            continue

        try:
            max_entries_for_feed = int(source_parameters["limit"])
        except (KeyError, ValueError):
            max_entries_for_feed = None

        # Retrieve updates from feeds.

        if source_parameters.get("type") == "url":
            try:
                feed_info, feed_updates = getUpdates(request, location, max_entries_for_feed, show_content)
                updates += feed_updates
                feeds.append((location, feed_info))
            except FeedMissingError:
                missing.append(location)
            except FeedContentTypeError:
                bad_content.append(location)

        # Retrieve updates from pages.

        elif source_parameters.get("type") == "page":
            page = Page(request, location)
            updates += getUpdatesFromPage(page, request)

            # Build feed-equivalent information for the update source.

            feeds.append((
                page.url(request, {"action" : "SharedUpdates", "doit" : "1"}), (
                    "internal", _("Updates from page %s") % location,
                    page.url(request)
                    )
                ))

        # Retrieve updates from message stores.

        elif source_parameters.get("type") == "store":
            page = Page(request, location)
            updates += getUpdatesFromStore(page, request)

            # Build feed-equivalent information for the update source.

            feeds.append((
                page.url(request, {"action" : "SharedUpdates", "store" : "1", "doit" : "1"}), (
                    "internal", _("Updates from message store on page %s") % location,
                    page.url(request)
                    )
                ))

    # Prepare the output.

    output = []
    append = output.append

    # Show the updates.

    if not show_content:
        append(fmt.bullet_list(on=1))

    # NOTE: Permit configurable sorting.

    updates.sort()
    updates.reverse()

    # Truncate the number of updates to the maximum number.

    updates = updates[:max_entries]

    for update in updates:

        # Emit content where appropriate.

        if show_content:
            append(fmt.div(on=1, css_class="moinshare-update"))

            if update.author:
                append(fmt.div(on=1, css_class="moinshare-author"))
                append(fmt.text(update.author))
                append(fmt.div(on=0))

            append(formatUpdate(update, request, fmt))

            append(fmt.div(on=1, css_class="moinshare-date"))
            append(fmt.text(str(update.updated)))
            append(fmt.div(on=0))

            append(fmt.div(on=0))

        # Or emit title and link information for items.

        elif update.title and update.link:
            append(fmt.listitem(on=1, css_class="moinshare-update"))
            append(fmt.url(on=1, href=update.link))
            append(fmt.icon('www'))
            append(fmt.text(" " + update.title))
            append(fmt.url(on=0))
            append(fmt.listitem(on=0))

    if not show_content:
        append(fmt.bullet_list(on=0))

    # Show the feeds.

    for feed_url, (feed_type, channel_title, channel_link) in feeds:
        if channel_title and channel_link:
            append(fmt.paragraph(on=1, css_class="moinshare-feed"))
            append(fmt.url(on=1, href=channel_link))
            append(fmt.text(channel_title))
            append(fmt.url(on=0))
            append(fmt.text(" "))
            append(fmt.url(on=1, href=feed_url))
            append(fmt.icon('rss'))
            append(fmt.url(on=0))
            append(fmt.paragraph(on=0))

    # Show errors.

    for feed_url in missing:
        append(fmt.paragraph(on=1, css_class="moinshare-missing-feed-error"))
        append(fmt.text(_("SharedContent: updates could not be retrieved for %s") % feed_url))
        append(fmt.paragraph(on=0))

    for feed_url in bad_content:
        append(fmt.paragraph(on=1, css_class="moinshare-content-type-feed-error"))
        return fmt.text(_("SharedContent: updates for %s were not provided in Atom or RSS format") % feed_url)
        append(fmt.paragraph(on=0))

    return ''.join(output)

# vim: tabstop=4 expandtab shiftwidth=4
